/*
    Copyright (c) Peter Sorotokin, 1998-2000
    See file "License.html" for terms of usage and
    redistribution.   
*/
package kmy.jint.lang;

/**
 *  A class that implements this interface is treated by Jint
 *  compiler as a "container". When some ("component") class
 *  is anonymously subclassed in a "container" class body, compiler
 *  will look for _add method in the "container" with "component"
 *  class as an argument. If such method is found it will be called
 *  right after new "component" object is created. There is no error
 *  if such method does not exist.
 *  <p>
 *  EXAMPLE:
 *  <p>
 *  jint code
 *  <p>
 *  <pre>
 *  class A implements AbstractContainer
 *  {
 *    void _add( Object x )
 *    {
 *    }
 *
 *    Object u
 *    {
 *    }
 *  }
 *  </pre>
 *
 *  is roughly equivalent to the following in Java:
 *
 *  <pre>  
 *  class A
 *  {
 *    void _add( Object x )
 *    {
 *    }
 *
 *    B u;
 *
 *    class B
 *    {
 *    }
 *
 *    A()
 *    {
 *      u = new B();
 *      add( u );
 *    }
 *  }
 *  </pre>
 */
public interface AbstractContainer
{
}
