/*
    Copyright (c) Peter Sorotokin, 1998-2000
    See file "License.html" for terms of usage and
    redistribution.   
*/
package kmy.jint.parser;

import kmy.jint.tree.ClassNode;
import kmy.jint.tree.MethodNode;
import kmy.jint.tree.MemberNode;
import kmy.jint.tree.BasicNode;
import kmy.jint.tree.IdentifierNode;

import kmy.jint.util.ErrorStorage;

/**
 *  This defines an object that "consumes" parsing tree as it is being produced.
 *  ClassConsumer object can be given to Parser and its methods will be called
 *  as corresponding constructions are being parsed.
 */
public class ClassConsumer
{

  /**
   *  Notifies class consumer when import statement is encountered.
   */
  public void tellImport( IdentifierNode imp )
  {
  }

  /**
   *  Notifies class consumer when package statement is encountered.
   */
  public void tellPackage( IdentifierNode pack )
  {
  }

  public void tellErrorStorage( ErrorStorage errorStorage )
  {
  }

  /**
   *  This method is called when a scope (class or method) is discovered by the parser.
   *  Note that scope is not just any {...}, it is either class or method (or [...] block
   *  in Jint which is considered to be an inner class with single method).  
   *  @param outerHanle handle to the outer class or null for top-level classes
   *  @param packageName name of the package, or null for inner classes
   *  @param name class name
   *  @returns handle to this class (to be passed as outerHandle to nested scopes)
   */
  public Object startScope( Object outerHandle, Identifier name, int flags,
	     IdentifierNode[] ext, IdentifierNode[] impl, Class what )
  {
    return outerHandle;
  }

  /**
   *  This method is called when scope (class or method) is completely parsed.
   */
  public void finishScope( Object handle, BasicNode def )
  {
  }
}
