/*
    Copyright (c) Peter Sorotokin, 1998-2000
    See file "License.html" for terms of usage and
    redistribution.   
*/
package kmy.jint.parser;

/**
 *  This class represents a simple (not qualified) identifier. All identifiers
 *  are listed in the NameTable and have a unique integer id. Thus, it is more
 *  efficient to compare and look up an Identifier then a String.
 */
public class Identifier
{
  /**
   *  Pointer to a NameTable where this Identifier is registered. Normally there
   *  is only one NameTable around.
   */
  public NameTable nameTable;

  /**
   *  String representation of this identifier.
   */
  public String name;

  /**
   *  Unique integer id for this identifier. All ids are negative.
   */
  public int    id;

  /**
   *  Constructor; meant to be used only by NameTable. Use NameTable.lookupAndInsert(String)
   *  if you want to get an Identifier. It will return the same identifier for all
   *  equal strings.
   */
  Identifier( NameTable owner, String name, int id )
  {
    nameTable = owner;
    this.name = name;
    this.id = id;
  }

  /**
   *  Hashcode is just identifier's id.
   */
  public int hashCode()
  {
    return id;
  }

  /**
   *  Idenifier is equal only to itself.
   */
  public boolean equals( Object t )
  {
    return this == t;
  }

  /**
   *  Some identifiers represent reserved keywords in Java and this
   *  method tests if this is one of these.
   */
  public boolean isReserved()
  {
    return (-id) < NameTable.reserved.length;
  }

  /**
   *  Returns identifier's string representation (its 'name' member).
   */
  public String toString()
  {
    return name;
  }
}
