/*
    Copyright (c) Peter Sorotokin, 1998-2000
    See file "License.html" for terms of usage and
    redistribution.   
*/
package kmy.jint.reflect;

import kmy.jint.parser.Identifier;
import kmy.jint.parser.QualifiedIdentifier;
import java.util.Hashtable;
import java.util.Enumeration;

import kmy.jint.constants.DefinitionConst;

abstract public class AbstractClass implements DefinitionConst
{
  abstract public String               getName();
  abstract public QualifiedIdentifier  getFullName();
  abstract public QualifiedIdentifier  getPackage();
  
  /**
   *  0 for Object, 1 for Object[], 2 for Object[][], etc.
   */
  public int getDimension()
  {
    return 0;
  }

  public AbstractClass getElementClass()
  {
    return null;
  }

  public Hashtable getImportCache()
  {
    return null;
  }

  abstract public int                  getModifiers();
  abstract public AbstractField        getField( Identifier name );
  abstract public Enumeration          getMethods( Identifier name, int nparam );
  abstract public AbstractClass[]      getInnerClasses();

  abstract public AbstractClass[]      getImplementedInterfaces();
  abstract public AbstractClass        getSuperclass();

  abstract public boolean              isSubclassOf( AbstractClass cls );

  abstract public boolean              isInterface();
  abstract public boolean              isPrimitive();

  abstract public String               getDescriptor();

  abstract public JintClassTable        getClassTable();

  public AbstractField getOuterField()
  {
    return null;
  }

  // also determines if a hidden argument is needed for constructor
  public AbstractClass getOuter()
  {
    return null;
  }

  public AbstractClass getLexicallyOuter()
  {
    return null;
  }

  private String jvmName;

  public String getJVMName()
  {
    if( jvmName == null )
      {
	String d = getDescriptor();
	if( d == null )
	  throw new RuntimeException( "Class " + getFullName() +
				      " ("+ getClass()+"): null descriptor" );  
	if( d.startsWith( "L" ) )
	  jvmName = d.substring( 1, d.length()-1 );
	else
	  jvmName = d;
      }
    return jvmName;
  }

  /**
   *  Returns number of JVM cells needed to hold this type. (long and double - 2,
   *  all others - 1).
   */
  public int getNCells()
  {
    return 1;
  }

  public boolean isWorld()
  {
    return false;
  }

  static final int FIELD_INDEX  = 0;
  static final int ARGLIST_INDEX = 1;
  static final int METHOD_INDEX = 2;

  public static final int VARIABLE = ARGLIST_INDEX - METHOD_INDEX;

  public String toString()
  {
    String cname = getClass().getName();
    int i = cname.lastIndexOf( '.' );
    if( i > 0 )
      cname = cname.substring( i + 1 );
    return cname + "[" + getDescriptor() + "]";
  }

}



