/*
    Copyright (c) Peter Sorotokin, 1998-2000
    See file "License.html" for terms of usage and
    redistribution.   
*/
package kmy.jint.reflect;

import kmy.jint.constants.DefinitionConst;

abstract public class AbstractMethod implements DefinitionConst
{
  String descriptor;
  String longDescriptor;
  
  boolean varargFlag;

  abstract public String getName();

  abstract public int             getModifiers();
  abstract public AbstractClass   getType();
  abstract public AbstractClass   getDeclaringClass();

  abstract public AbstractClass[] getExceptionTypes();
  abstract public AbstractClass[] getParameterTypes();

  public static final int SAME          = 0;
  public static final int MORE_SPECIFIC = 1;
  public static final int LESS_SPECIFIC = 2;
  public static final int UNDEFINED     = 3;

  public AbstractMethod()
  {
  }

  public static String makeStubDescriptor( String name, AbstractClass[] param )
  {
    return name + makeStubDescriptor( param );
  }

  static String makeStubDescriptor( AbstractClass[] param )
  { 
    StringBuffer sb = new StringBuffer();
    sb.append( '(' );
    if( param != null )
      {
	for( int i = 0 ; i < param.length ; i++ )
	  {
	    String dscr = param[i].getDescriptor();
	    sb.append( dscr );
	  }
      }
    sb.append( ')' );
    return sb.toString();
  }

  protected void initDescriptor()
  {
    if( descriptor == null )
      descriptor = (makeStubDescriptor( getParameterTypes() ) + 
		    getType().getDescriptor()).intern();
    if( longDescriptor == null )
      longDescriptor = (getName().toString() + descriptor).intern();
  }

  public String getTypeDescriptor()
  {
    if( descriptor == null )
      initDescriptor();
    return descriptor;
  }

  public String getDescriptor()
  {
    if( longDescriptor == null )
      initDescriptor();
    return longDescriptor;
  }

  public String toString()
  {
    String cname = getClass().getName();
    int i = cname.lastIndexOf( '.' );
    if( i > 0 )
      cname = cname.substring( i + 1 );
    return cname + "[" + getDeclaringClass() + "." + getDescriptor() + "]";
  }

  public String getAccessor()
  {
    return null;
  }
}

