/*
    Copyright (c) Peter Sorotokin, 1998-2000
    See file "License.html" for terms of usage and
    redistribution.   
*/
package kmy.jint.reflect;

import kmy.jint.parser.*;
import java.util.*;
import kmy.jint.tree.DefinitionNode;

public class AnyClass extends AbstractClass
{

  QualifiedIdentifier fullName;
  JintClassTable classTable;
  QualifiedIdentifier packageName;

  public AnyClass( JintClassTable ct )
  {
    classTable = ct;
    NameTable nameTable = ct.getNameTable();
    Identifier[] list = { nameTable.KMY, nameTable.JINT, nameTable.LANG, nameTable.ANY };
    fullName = new QualifiedIdentifier( list );
    packageName = fullName.dropOne();
    fullName.setType( this );
  }

  public String getName()
  {
    return "Any";
  }

  public QualifiedIdentifier  getFullName()
  {
    return fullName;
  }
  
  public QualifiedIdentifier getPackage()
  {
    return packageName;
  }

  public AbstractClass getElementClass()
  {
    return this;
  }

  public int getModifiers()
  {
    return FINAL_FLAG | ABSTRACT_FLAG | PUBLIC_FLAG;
  }

  public AbstractField getField( Identifier name )
  {
    return new AnyField( classTable, name );
  }

  public Enumeration getMethods( Identifier name, int nparam )
  {
    if( nparam == VARIABLE )
      return null;
    Vector methods = new Vector();
    methods.addElement( new AnyMethod( classTable, name, nparam ) );
    return methods.elements();
  }
  
  public AbstractClass[] getInnerClasses()
  {
    return null;
  }

  public AbstractClass[] getImplementedInterfaces()
  {
    return null;
  }

  public AbstractClass getSuperclass()
  {
    return classTable.objectClass;
  }

  public boolean isSubclassOf( AbstractClass cls )
  {
    return false;
  }

  public boolean isInterface()
  {
    return false;
  }

  public boolean isPrimitive()
  {
    return false;
  }
  
  public String getDescriptor()
  {
    return "A";
  }

  public String getJVMName()
  {
    return "Any";
  }

  public int getNCells()
  {
    return 1;
  }

  public boolean isWorld()
  {
    return true;
  }

  public JintClassTable getClassTable()
  {
    return classTable;
  }

}
