/*
    Copyright (c) Peter Sorotokin, 1998-2000
    See file "License.html" for terms of usage and
    redistribution.   
*/
package kmy.jint.reflect;

import java.lang.reflect.*;
import java.util.*;

import kmy.jint.parser.*;

public class ArrayClass extends AbstractClass
{
  JintClassTable classTable;
  AbstractClass baseClass;
  AbstractClass elementClass;
  int           dim;
  String        descriptor;

  QualifiedIdentifier name;
  
  ArrayClass( JintClassTable classTable, AbstractClass baseClass, QualifiedIdentifier name )
  {
    if( !name.base().equals( baseClass.getFullName() ) )
      throw new IllegalArgumentException( "Name conflicts with baseClass: " + name + " " + baseClass );
    this.classTable = classTable;
    this.baseClass = baseClass;

    dim = name.getDimension();
    if( dim == 0 )
      throw new IllegalArgumentException( "Cannot create array class with number of dimensions 0" );

    StringBuffer buf = new StringBuffer();
    for( int i = 0 ; i < dim ; i++ )
      buf.append( '[' );
    buf.append( baseClass.getDescriptor() );
    descriptor = buf.toString();

    name.setType( this );
    this.name = name;
  }

  public String getName()
  {
    StringBuffer sb = new StringBuffer();
    sb.append( baseClass.getName() );
    for( int i = 0 ; i < dim ; i++ )
      sb.append( "[]" );
    return sb.toString();
  }

  public QualifiedIdentifier getPackage()
  {
    return baseClass.getPackage();
  }

  public QualifiedIdentifier getFullName()
  {
    return name;
  }

  /**
   *  0 for Object, 1 for Object[], 2 for Object[][], etc.
   */
  public int getDimension()
  {
    return dim;
  }

  public int getModifiers()
  {
    return baseClass.getModifiers();
  }

  public AbstractField getField( Identifier name )
  {
    return null;
  }

  public Enumeration getMethods( Identifier name, int nparam )
  {
    return null;
  }

  public AbstractClass[] getInnerClasses()
  {
    return null;
  }

  public AbstractClass[] getImplementedInterfaces()
  {
    return null;
  }

  public AbstractClass getSuperclass()
  {
    return classTable.objectClass;
  }

  /**
   *  Check if object of class this can be assigned to variable of class cls
   */
  public boolean isSubclassOf( AbstractClass cls )
  {
    if( cls instanceof ArrayClass )
      {
	ArrayClass other = (ArrayClass)cls;
	int dimDiff = other.getDimension() - dim;
	if( dimDiff > 0 ) // other is deeper
	  return false; 
	if( dimDiff == 0 )
	  return baseClass.isSubclassOf( other.getBaseClass() );
	cls = other.getBaseClass();
      }
    return cls == classTable.objectClass || cls == classTable.serializableClass ||
           cls == classTable.cloneableClass ;
  }

  public boolean isInterface()
  {
    return false;
  }

  public boolean isPrimitive()
  {
    return false;
  }

  public JintClassTable getClassTable()
  {
    return classTable;
  }

  public String getDescriptor()
  {
    return descriptor;
  }

  public AbstractClass getElementClass()
  {
    if( elementClass == null )
      elementClass = classTable.getClass( 
	   new QualifiedIdentifier( baseClass.getFullName(), dim-1 ) );
    return elementClass;
  }

  public AbstractClass getBaseClass()
  {
    return baseClass;
  }

}
