/*
    Copyright (c) Peter Sorotokin, 1998-2000
    See file "License.html" for terms of usage and
    redistribution.   
*/
package kmy.jint.reflect;

import kmy.jint.parser.QualifiedIdentifier;
import kmy.jint.parser.Identifier;
import kmy.jint.parser.NameTable;
import kmy.jint.gen.JVMClassReader;
import kmy.jint.gen.JVMClassConsumer;
import kmy.jint.constants.TokenConst;

import kmy.jint.lang.ArgList;

import java.io.*;

import java.util.StringTokenizer;

public class CompiledClass extends BasicClass implements TokenConst
{

  static final Object NOTCONST = new Object();

  static String varArgSignature = "L" + ArgList.class.getName().replace( '.', '/' ) + ";)";

  public CompiledClass( JintClassTable ct,
		    AbstractClass outer, InputStream in ) throws IOException
  {
    super( ct, null, outer );
    if( ! (in instanceof BufferedInputStream) && ! (in instanceof ByteArrayInputStream) )
      in = new BufferedInputStream( in );
    Consumer consumer = new Consumer();
    JVMClassReader reader = new JVMClassReader( consumer );
    reader.parse( new DataInputStream( in ) );
    initDescriptor();
  }

  class Consumer extends JVMClassConsumer
  {

    private int    flags;
    private String name;
    private String type;
    private String[] exceptions;
    private Object value;

    private int countParameters( String type )
    {
      int nparam = 0;
      if( type.charAt(0) != '(' )
	throw new RuntimeException( "Malformed method descriptor: " + type );
      int i = 1;
      while( true )
	{
	  switch( type.charAt(i++) )
	    {
	      case ')' :
		return nparam;
	      case 'I' :
	      case 'S' :
	      case 'Z' :
	      case 'B' :
	      case 'C' :
	      case 'F' :
	      case 'D' :
	      case 'J' :
	      case 'A' :
		break;
	      case 'L' :
		i = type.indexOf( ';', i ) + 1;
		if( i <= 0 )
		  throw new RuntimeException( "Malformed method descriptor: " + type );
		
		break;
	      case '[' :
		while( true )
		  {
		    if( i >= type.length() )
		      throw new RuntimeException( "Malformed method descriptor: " + type );
		    if( type.charAt(i) == '[' )
		      i++;
		    else
		      break;
		  }
		if( type.charAt(i) == 'L' )
		  {
		    i = type.indexOf( ';', i ) + 1;
		    if( i > 0 )
		      break;
		  }
		else
		  {
		    i++;
		    break;
		  }
		// fall thru
	      default:
		throw new RuntimeException( "Malformed method descriptor: " + type );
	    }
	  nparam++;
	}
    }

    public void startClass( int flags, String name, String superName )
    {
      modifiers = flags;
      StringTokenizer tok = new StringTokenizer( name, "$/" );
      Identifier[] arr = new Identifier[tok.countTokens()];
      NameTable nameTable = classTable.getNameTable();
      for( int i = 0 ; i < arr.length ; i++ )
	arr[i] = nameTable.lookupAndInsert( tok.nextToken() );
      className = new QualifiedIdentifier( arr );
      superclass = classTable.getClass( "L" + superName + ";" );
    }

    public void tellInterfaces( String[] iNames )
    {
      if( iNames == null )
	return;
      interfaces = new AbstractClass[iNames.length];
      for( int i = 0 ; i < interfaces.length ; i++ )
	interfaces[i] = classTable.getClass( "L" + iNames[i] + ";" );
    }

    public void startField( int flags, String name, String type )
    {
      this.flags = flags;
      this.name = name;
      this.type = type;
      this.value = NOTCONST;
      this.exceptions = null;
    }

    public boolean wantFieldAttribute( String name )
    {
      return name == "ConstantValue" || name == "JintType" ;
    }

    public void tellFieldAttribute( String name, Object value )
    {
      if( name == "ConstantValue" )
	{
	  flags |= CONST_FLAG;
	  this.value = value;
	}
      else if( name == "JintType" )
	this.type = (String)value;
    }

    public void endField()
    {
      addMember( classTable.getNameTable().lookupAndInsert( name ), 
		 new CompiledField( flags, CompiledClass.this, name, type, value ),
		 FIELD_INDEX );
    }

    public void startMethod( int flags, String name, String type )
    {
      this.flags = flags;
      this.name = name;
      this.type = type;
      this.exceptions = null;
    }

    public boolean wantMethodAttribute( String name )
    {
      return name == "Exceptions" || name == "JintType" ;
    }

    public void tellMethodAttribute( String name, Object value )
    {
      if( name == "Exceptions" )
	this.exceptions = (String[])value;
      else if( name == "JintType" )
	this.type = (String)value;
    }

    public void endMethod()
    {
      int nargs;
      if( type.indexOf( varArgSignature ) >= 0 )
	nargs = ARGLIST_INDEX;
      else
	nargs = METHOD_INDEX + countParameters( type );
      addMember( classTable.getNameTable().lookupAndInsert( name ),
		 new CompiledMethod( flags, CompiledClass.this, name, type, exceptions ),
		 nargs );
    }  

    public void endClass()
    {
    }
    
  }
}

