/*
    Copyright (c) Peter Sorotokin, 1998-2000
    See file "License.html" for terms of usage and
    redistribution.   
*/
package kmy.jint.reflect;

import java.util.Vector;

public class CompiledMethod extends AbstractMethod
{

  int flags;
  String name;
  CompiledClass clazz;
  AbstractClass type;

  AbstractClass[] parameterTypes;
  AbstractClass[] exceptionTypes;

  String[] exceptionTypeNames;

  static AbstractClass[] EMPTY = {};

  CompiledMethod( int flags, CompiledClass clazz, String name,
		  String descriptor, String[] exceptionTypeNames )
  {
    super();
    this.descriptor = descriptor;
    this.clazz = clazz;
    this.flags = flags;
    this.name = name;
    this.exceptionTypeNames = exceptionTypeNames;
  }

  public String getName()
  {
    return name;
  }

  public int getModifiers()
  {
    return flags;
  }

  public AbstractClass getType()
  {
    if( type == null )
      {
	String retType = getTypeDescriptor();
	retType = retType.substring( retType.indexOf( ')' ) + 1 );
	type = clazz.classTable.getClass( retType );
      }
    return type;
  }

  public AbstractClass getDeclaringClass()
  {
    return clazz;
  }

  public AbstractClass[] getExceptionTypes()
  {
    if( exceptionTypes == null )
      {
	if( exceptionTypeNames == null || exceptionTypeNames.length == 0 )
	  exceptionTypes = EMPTY;
	else
	  {
	    exceptionTypes = new AbstractClass[exceptionTypeNames.length];
	    for( int i = 0 ; i < exceptionTypeNames.length ; i++ )
	      exceptionTypes[i] = clazz.classTable.getClass( "L" + exceptionTypeNames[i] + ";" );
	  }
      }
    return exceptionTypes;
  }

  public AbstractClass[] getParameterTypes()
  {
    if( parameterTypes == null )
      {
	Vector types = new Vector();
	String type = getTypeDescriptor();
	if( type.charAt(0) != '(' )
	  throw new RuntimeException( "Malformed method descriptor: " + type );
	int i = 1;
	int prev = 1;
  LOOP: while( true )
          {
	    switch( type.charAt(i++) )
	      {
	        case ')' :
		  break LOOP;
	        case 'I' :
		case 'S' :
		case 'Z' :
		case 'B' :
		case 'C' :
		case 'F' :
		case 'D' :
		case 'J' :
		case 'A' :
		  break;
		case 'L' :
		  i = type.indexOf( ';', i ) + 1;
		  if( i <= 0 )
		    throw new RuntimeException( "Malformed method descriptor: " + type );
		  
		  break;
		case '[' :
		  while( true )
		    {
		      if( i >= type.length() )
			throw new RuntimeException( "Malformed method descriptor: " + type );
		      if( type.charAt(i) == '[' )
			i++;
		      else
			break;
		    }
		  if( type.charAt(i) == 'L' )
		    {
		      i = type.indexOf( ';', i ) + 1;
		      if( i > 0 )
			break;
		    }
		  else
		    {
		      i++;
		      break;
		    }
		  // fall thru
		default:
		  throw new RuntimeException( "Malformed method descriptor: " + type );
	      }
	     types.addElement( clazz.classTable.getClass(type.substring( prev, i )) );
	     prev = i;
	  }
	if( types.size() == 0 )
	  parameterTypes = EMPTY;
	else
	  {
	    parameterTypes = new AbstractClass[types.size()];
	    types.copyInto( parameterTypes );
	  }
      }
    return parameterTypes;
  }

}
