/*
    Copyright (c) Peter Sorotokin, 1998-2000
    See file "License.html" for terms of usage and
    redistribution.   
*/
package kmy.jint.reflect;

import kmy.jint.parser.Identifier;
import kmy.jint.parser.QualifiedIdentifier;

public class JintMethod extends AbstractMethod
{
  String          name;

  JintClass       declaring;

  AbstractClass   type;
  AbstractClass[] params;
  AbstractClass[] exceptions;

  int             modifiers;

  QualifiedIdentifier typeName;
  QualifiedIdentifier[] paramTypeNames;
  QualifiedIdentifier[] exceptionNames;

  JintMethod( JintClassTable classTable, JintClass declaring,
	      int modifiers, Identifier name,
	      QualifiedIdentifier type,
	      QualifiedIdentifier[] params,
	      QualifiedIdentifier[] exceptions )
  {

    this.declaring = declaring;

    if( name == classTable.getNameTable().INIT )
      this.type = classTable.voidType;
    else if( type == null )
      this.type = classTable.anyClass;

    typeName = type;

    paramTypeNames = params;
    exceptionNames = exceptions;

    this.modifiers = modifiers;

    this.name = name.toString();

    if( (modifiers & SCRIPT_STYLE_FLAG) != 0 && 
	(exceptions == null || exceptions.length == 0) )
      {
	exceptionNames = new QualifiedIdentifier[1];
	exceptionNames[0] = classTable.exceptionClass.getFullName();
      }    
  }

  protected void initDescriptor()
  { 
    if( descriptor != null )
      return;
    StringBuffer sb = new StringBuffer();
    sb.append( '(' );
    if( paramTypeNames != null )
      {
	for( int i = 0 ; i < paramTypeNames.length ; i++ )
	  if( paramTypeNames[i] != null )
	    paramTypeNames[i].writeDescriptor( sb );
	  else
	    sb.append( declaring.getClassTable().anyClass.getDescriptor() );
      }
    sb.append( ')' );
    if( name == "<init>" )
      sb.append( "V" );
    else if( typeName == null )
      sb.append( declaring.getClassTable().anyClass.getDescriptor() );
    else
      typeName.writeDescriptor( sb );
    descriptor = sb.toString();
    super.initDescriptor();
  }

  public String getName()
  {
    return name;
  }

  public int getModifiers()
  {
    return modifiers;
  }

  public AbstractClass getDeclaringClass()
  {
    return declaring;
  }

  public AbstractClass getType()
  {
    if( type == null )
      type = declaring.getClassTable().getClass( typeName );
    return type;
  }

  public AbstractClass[] getExceptionTypes()
  {
    if( exceptions == null && exceptionNames != null )
      {
	JintClassTable classTable = declaring.getClassTable();
	exceptions = new AbstractClass[exceptionNames.length];
	for( int i = 0 ; i < exceptionNames.length ; i++ )
	  exceptions[i] = classTable.getClass( exceptionNames[i] );
      }
    return exceptions;
  }

  public AbstractClass[] getParameterTypes()
  {
    if( params == null && paramTypeNames != null )
      {
	JintClassTable classTable = declaring.getClassTable();
	params = new AbstractClass[paramTypeNames.length];
	for( int i = 0 ; i < paramTypeNames.length ; i++ )
	  if( paramTypeNames[i] == null )
	    params[i] = classTable.anyClass;
	  else
	    params[i] = classTable.getClass( paramTypeNames[i] );
      }
    return params;
  }
}
