/*
    Copyright (c) Peter Sorotokin, 1998-2000
    See file "License.html" for terms of usage and
    redistribution.   
*/
package kmy.jint.tools;

import java.io.*;
import kmy.jint.gen.JVMClassReader;
import kmy.jint.gen.JVMClassConsumer;

public class Disassembler
{

  static class Consumer extends JVMClassConsumer
    {
      public void tellVersion( int version, int subversion )
      {
      }
      
      public void startClass( int flags, String name, String superName )
      {
	System.out.println( "\tclass " + Integer.toHexString(flags) + " " + name );
	if( superName != null )
	  System.out.println( "\textends " + superName );
      }
      
      public void tellInterfaces( String[] iNames )
      {
	for( int i = 0 ; i < iNames.length ; i++ )
	  System.out.println( "\timplements " + iNames[i] );
      }
      
      public void startField( int flags, String name, String type )
      {
	System.out.print( "\tfield " + Integer.toHexString(flags) + 
			 " " + name + " " + type );
      }
      
      public boolean wantFieldAttribute( String name )
      {
	return (name == "ConstantValue" );
      }
      
      public void tellFieldAttribute( String name, Object value )
      {
	if( name == "ConstantValue" )
	  System.out.print( " = " + value.getClass().getName() + " " + value );
	else
	  {
	    System.out.println();
	    System.out.print( "\t\tattribute " + name + " " + value );
	  }
      }
      
      public void endField()
      {
	System.out.println();
      }
      
      public void startMethod( int flags, String name, String type )
      {
	System.out.println( "\tmethod " + Integer.toHexString(flags) +
			   " " + name + " " + type );
      }
      
      public boolean wantMethodAttribute( String name )
      {
	return true;
      }
      
      public void tellMethodAttribute( String name, Object value )
      {
	if( name == "Exceptions" )
	  {
	    String[] list = (String[])value;
	    for( int i = 0 ; i < list.length ; i++ )
	      System.out.println( "\tthrows " + list[i] );
	  }
	else
	  {
	    if( value instanceof byte[] )
	      {
		byte[] arr = (byte[])value;
		StringBuffer sb = new StringBuffer();
		for( int i = 0 ; i < 30 && i < arr.length ; i++ )
		  {
		    sb.append( Integer.toHexString((arr[i]>>8)&0xF) );
		    sb.append( Integer.toHexString((arr[i])&0xF) );
		  }
		System.out.print( "\tattribute " + name + ": byte[" + arr.length + "]" );
		if( arr.length < 10 )
		  System.out.print( " " );
		else
		  {
		    System.out.println();
		    System.out.print( "\t  " );
		  }
		System.out.println( sb.toString() );
	      }
	    else
	      System.out.println( "\tattribute " + name + " " + value );
	  }
      }
      
      public void endMethod()
      {
	System.out.println( "\tend" );
      }  
      
      public void endClass()
      {
	System.out.println( "\tendclass" );
      }
      
    }

  public static void main( String[] args )
  {
    try
      {
	JVMClassReader reader = new JVMClassReader( new Consumer() );
	InputStream in = new FileInputStream( args[0] );
	reader.parse( in );
      }
    catch( Exception e )
      {
	e.printStackTrace();
      }
  }
  
}
