/*
    Copyright (c) Peter Sorotokin, 1998-2000
    See file "License.html" for terms of usage and
    redistribution.   
*/
package kmy.jint.tree;

import kmy.jint.parser.AbstractTokenizer;

import java.io.CharArrayWriter;
import java.io.PrintWriter;

/**
 *  This is a base class representing parsing tree node; all other
 *  nodes are derived from it.
 */
public abstract class BasicNode
{

  private int     filePos;

  /**
   *  Attachment is a general-purpose field that can hold some
   *  useful information generated by some kind of parsing tree
   *  processing (like semantic analysis). This is somewhat
   *  sloppy design, but that's how it is.
   */
  public  Object  attachment;

  /**
   *  Initialize BasicNode that represents something at position
   *  <i>filePos</i> is the input file.
   */
  public BasicNode( int filePos )
  {
    this.filePos = filePos;
  }

  /**
   *  Returns file position where the element what this node represents
   *  was located.
   */
  public int getFilePos()
  {
    return filePos;
  }

  /**
   *  Returns line where the element what this node represents
   *  was located.
   */
  public int getLine()
  {
    return AbstractTokenizer.getLine( filePos );
  }

  /**
   *  Returns line's column where the element what this node represents
   *  was located.
   */
  public void setFilePos( int filePos )
  {
    this.filePos = filePos;
  }

  /**
   *  Assigns this node attachment.
   */
  public void setAttachment( Object obj )
  {
    attachment = obj;
  }

  /**
   *  Returns this node attachment.
   */
  public Object getAttachment()
  {
    return attachment;
  }

  /**
   *  Returns string representation of this node.
   */
  public String toString()
  {
    PrintContext pc = new PrintContext();
    eval( pc );
    return pc.toString();
  }

  /**
   *  Calls corresponding eval* method in the given Context.
   */
  abstract public Object eval( Context context );

}
