/*
    Copyright (c) Peter Sorotokin, 1998-2000
    See file "License.html" for terms of usage and
    redistribution.   
*/
package kmy.jint.tree;

import kmy.jint.parser.Identifier;

public class IdentifierNode extends ExpressionNode
{

  public Identifier[]     idList;
  public ExpressionNode[] indexes;

  public IdentifierNode( int filePos, Identifier[] idList )
  {
    super( filePos );
    this.idList = idList;
  }

  public IdentifierNode( int filePos, Identifier id )
  {
    super( filePos );
    idList = new Identifier[1];
    idList[0] = id;
  }

  public IdentifierNode( IdentifierNode orig, ExpressionNode[] indexes )
  {
    super( orig.getFilePos() );
    this.idList = orig.idList;
    if( orig.indexes == null )
      this.indexes = indexes;
    else
      {
	this.indexes = new ExpressionNode[ indexes.length + orig.indexes.length ];
	System.arraycopy( orig.indexes, 0, this.indexes, 0, orig.indexes.length );
	System.arraycopy( indexes, 0,
			  this.indexes, orig.indexes.length, indexes.length );
      }
  }

  public IdentifierNode( int filePos, Identifier[] idList, ExpressionNode[] indexes )
  {
    super( filePos );
    this.idList = idList;
    this.indexes = indexes;
  }

  public Object eval( Context context )
  {
    return context.evalIdentifier( this );
  }

  public int hashCode()
  {
    int r = 0;
    for( int i = 0 ; i < idList.length ; i++ )
      if( idList[i] != null )
	r += idList[i].hashCode();
    return r;
  }

  public boolean equals( Object other )
  {
    if( other == null )
      return false;
    if( other.getClass() != IdentifierNode.class )
      return false;
    IdentifierNode otherId = (IdentifierNode)other; 
    if( otherId.idList.length != idList.length )
      return false;
    for( int i = 0 ; i < idList.length ; i++ )
      if( idList[i] != otherId.idList[i] )
	return false;
    return true;
  }
}
