/*
    Copyright (c) Peter Sorotokin, 1998-2000
    See file "License.html" for terms of usage and
    redistribution.   
*/
package kmy.jint.tree;

import kmy.jint.constants.TokenConst;
import kmy.jint.constants.ErrorCodes;
import kmy.jint.util.AnyUtilities;
import kmy.jint.util.CompilerException;

public class InfixNode extends ExpressionNode implements TokenConst, ErrorCodes
{

  public int op;
  public ExpressionNode arg1;
  public ExpressionNode arg2;

  public InfixNode( int filePos, int op, ExpressionNode arg1, ExpressionNode arg2 )
  {
    super( filePos );
    this.op = op;
    this.arg1 = arg1;
    this.arg2 = arg2;
  }

  public Object eval( Context context )
  {
    return context.evalInfix( this );
  }

  public String toString()
  {
    if( op == '[' )
      if( arg2 == null )
	return arg1.toString() + "[]";
      else
	return arg1.toString() + "[" + arg2 + "]";
    else
      return "(" + arg1.toString() + " " + (char)op + " " + arg2.toString() + ")";
  }

  public Object getConstValue()
  {
    try
      {
	Object v1 = arg1.getConstValue();
	if( v1 == NOT_CONST )
	  return v1;
	Object v2 = arg2.getConstValue();
	if( v2 == NOT_CONST )
	  return v2;
	if( op == '+' || op == '-' || op == '*' || op == '/' ||
	    op == '&' || op == '|' || op == '^' || op == '%' )
	  return AnyUtilities.op( op, v1, v2 );
	if( op == '>' || op == '<' || 
	    op == TOKEN_GE || op == TOKEN_LE || op == TOKEN_EE || op == TOKEN_NE )
	  return AnyUtilities.cond( op, v1, v2 ) ? Boolean.TRUE : Boolean.FALSE;
	if( (op == TOKEN_GG || op == TOKEN_LL || op == TOKEN_GGG) )
	  if( v2 instanceof Integer || v2 instanceof Byte || v2 instanceof Short || v2 instanceof Long )
	    return AnyUtilities.shift( op, v1, ((Number)v2).intValue() );
	  else if( v2 instanceof Character )
	    return AnyUtilities.shift( op, v1, ((Character)v2).charValue() );
      }
    catch( CompilerException e )
      {
	throw e;
      }
    catch( RuntimeException e )
      {
	throw new CompilerException( ERR_A_CONSTERROR, getFilePos(), e );
      }
    return NOT_CONST;
  }

}
