/*
    Copyright (c) Peter Sorotokin, 1998-2000
    See file "License.html" for terms of usage and
    redistribution.   
*/
package kmy.jint.util;

import java.util.Vector;
import java.util.Hashtable;

public class DispatchObject implements Dispatch
{

  public static final Object NULL = new Object() { public String toString() { return "NULL"; } };
  private static final Class[] EMPTY = {};
  private static final Class[] OBJECT = { Object.class };

  Vector      targets = new Vector();
  Hashtable   softFields = new Hashtable();

  public DispatchObject()
  {
    targets.addElement( this );
  }

  private synchronized Object[] getTargets()
  {
    Object[] arr = new Object[targets.size()];
    targets.copyInto( arr );
    return arr;
  }

  public Object callMethod( String name, Class[] types, Object[] args, int cacheId )
    throws Exception
  {
    Object[] t = getTargets();
    String origName = name;
    for( int i = 0 ; i < t.length ; i++ )
      {
	Object target = t[i];	  
	try
	  {
	    if( target == this && ( name == "toString" || name == "hashCode"
				    || name == "clone" || name == "equals" ) )
	      return AnyUtilities.callMethod0( this, getClass(), "__" + name, types, args, cacheId );
	    else
	      return AnyUtilities.callMethod0( target, target.getClass(), name, types, args, cacheId );
	  }
	catch( DispatchException e )
	  {
	  }
      }
    throw new DispatchException( "Method not found: " + name );
  }

  public void setField( Object value, String name, int cacheId )
    throws DispatchException
  {
    Object[] t = getTargets();
    for( int i = 0 ; i < t.length ; i++ )
      try
	{
	  AnyUtilities.setField0( t[i], value, name, cacheId );
	  return;
	}
      catch( DispatchException e )
	{
	}
    if( value == NULL )
      softFields.remove( name );
    else if( value == null )
      softFields.put( name, NULL );
    else
      softFields.put( name, value );
  }
  
  public Object getField( String name, int cacheId )
    throws DispatchException
  {
    Object[] t = getTargets();
    for( int i = 0 ; i < t.length ; i++ )
      try
	{
	  return AnyUtilities.getField0( t[i], name, cacheId );
	}
      catch( DispatchException e )
	{
	}
    Object res = softFields.get( name );
    if( res == null )
      throw new DispatchException( "Field not found: " + getClass().getName() + "." + name );
    if( res == NULL )
      return null;
    return res;
  }

  public Object getElement( Object name )
  {
    Object res = softFields.get( name );
    if( res == null )
      throw new RuntimeException( "No element '" + name + "' found in this object" );
    if( res == NULL )
      return null;
    return res;
  }
  
  public void setElement( Object name, Object value )
  {
    if( value == NULL )
      softFields.remove( name );
    else if( value == null )
      softFields.put( name, NULL );
    else
      softFields.put( name, value );
  }

  public void addTarget( Object obj )
  {
    targets.insertElementAt( obj, 0 );
  }

  public void removeTarget( Object obj )
  {
    targets.removeElement( obj );
  }

  public String toString()
  {
    try
      {
	return (String)callMethod( "toString", EMPTY, EMPTY, 1 );
      }
    catch( RuntimeException e )
      {
	throw e;
      }
    catch( Exception e )
      {
	throw new RuntimeException( e.toString() );
      }
  }

  public int hashCode()
  {
    try
      {
	return ((Number)callMethod( "hashCode", EMPTY, EMPTY, 2 )).intValue();
      }
    catch( RuntimeException e )
      {
	throw e;
      }
    catch( Exception e )
      {
	throw new RuntimeException( e.toString() );
      }
  }

  public boolean equals( Object other )
  {
    try
      {
	Object[] args = { other };
	return ((Boolean)callMethod( "equals", OBJECT, args, 3 )).booleanValue();
      }
    catch( RuntimeException e )
      {
	throw e;
      }
    catch( Exception e )
      {
	throw new RuntimeException( e.toString() );
      }
  }

  public Object clone()
  {
    try
      {
	return callMethod( "clone", EMPTY, EMPTY, 4 );
      }
    catch( RuntimeException e )
      {
	throw e;
      }
    catch( Exception e )
      {
	throw new RuntimeException( e.toString() );
      }
  }

  public String __toString()
  {
    return super.toString();
  }

  public int __hashCode()
  {
    return super.hashCode();
  }

  public boolean __equals( Object other )
  {
    return super.equals( other );
  }

  public Object __clone()
  {
    throw new RuntimeException( "Not supported!" );
  }

}
