/*
    Copyright (c) Peter Sorotokin, 1998-2000
    See file "License.html" for terms of usage and
    redistribution.   
*/
package kmy.jint.util;

import java.util.Vector;
import java.util.Enumeration;
import java.io.BufferedReader;
import java.io.IOException;

import kmy.jint.constants.ErrorCodes;

public class ErrorKeeper implements ErrorStorage, ErrorCodes
{

  Vector errors = new Vector();

  boolean stackTrace = false;

  public void storeError( CompilerException e )
  {
    errors.addElement( e );
  }

  public void setStackTrace( boolean st )
  {
    stackTrace = st;
  }

  public boolean hasErrors()
  {
    synchronized( errors )
    {
      Enumeration list = errors.elements();
      while( list.hasMoreElements() )
	{
	  CompilerException e = (CompilerException)list.nextElement();
	  if( (e.getCode() & ERR_MASK_SEVERITY) >= ERR_MASK_ERROR )
	    return true;
	}
    }
    return false;
  }

  public boolean hasMessages()
  {
    return errors.size() != 0;
  }

  private static void sortErrors( CompilerException[] arr )
  {
    for( int i = 0 ; i < arr.length ; i++ )
      {
	int filePos = arr[i].getFilePos()>>>1;// so it's not negative
	for( int j = i+1 ; j < arr.length ; j++ )
	  {
	    int p = arr[j].getFilePos()>>>1;
	    if( p < filePos )
	      {
		CompilerException tmp = arr[i];
		arr[i] = arr[j];
		arr[j] = tmp;
		filePos = p;
	      }
	  }
      }
  }

  public void clean()
  {
    errors.removeAllElements();
  }

  public void report( String name, BufferedReader in )
  {
    if( errors.size() == 0 )
      return;
    CompilerException[] err = new CompilerException[ errors.size() ];
    errors.copyInto( err );
    sortErrors( err );
    int fileLine = 0;
    String str = null;
    for( int i = 0 ; i < err.length ; i++ )
      {
	int line = err[i].getLine();
	int col = err[i].getColumn() - 1;
	String code = Integer.toHexString(err[i].getCode());
	if( line < 4000000 )
	  System.err.println( name + ": " + line + ": [" + code + "] "  + err[i].getMessage() );
	else
	  System.err.println( name + ": <???>: [" + code + "] " + err[i].getMessage() );
	if( in != null )
	  try
	    {
	      while( line > fileLine )
		{
		  str = in.readLine();
		  fileLine++;
		  if( str == null )
		    break;
		}
	      if( str != null )
		{
		  System.err.println( str );
		  int strl = str.length();
		  for( int k = 0 ; k < col ; k++ )
		    if( k < strl && str.charAt(k) == '\t' )
		      System.err.print( '\t' );
		    else
		      System.err.print( ' ' );
		  System.err.println( '^' );
		  
		}
	    }
	  catch( IOException e )
	    {
	    }
	if( stackTrace || (err[i].getCode() & ERR_MASK_SEVERITY) >= ERR_MASK_INTERNAL )
	  err[i].printStackTrace();
      }
  }

}
