/*
    Copyright (c) Peter Sorotokin, 1998-2000
    See file "License.html" for terms of usage and
    redistribution.   
*/
package kmy.jint.util;

import java.lang.reflect.*;
import kmy.jint.lang.ArgList;
import java.util.Enumeration;

public class JVMClassMethods extends ClassMethods
{

  static Class intType = Integer.TYPE;
  static Class shortType = Short.TYPE;
  static Class byteType = Byte.TYPE;
  static Class longType = Long.TYPE;
  static Class charType = Character.TYPE;
  static Class floatType = Float.TYPE;
  static Class doubleType = Double.TYPE;
  static Class booleanType = Boolean.TYPE;
  static Class voidType = Void.TYPE;

  public JVMClassMethods()
  {
  }

  public Object getSuperclass( Object clazz )
  {
    return ((Class)clazz).getSuperclass();
  }

  public Object[] getImplementedInterfaces( Object clazz )
  {
    return ((Class)clazz).getInterfaces();
  }

  public Object getArgListClass()
  {
    return ArgList.class;
  }

  public Object getAnyClass()
  {
    return null;
  }

  /**
   *  Returns object used to name constructors ("<init>").
   */
  public Object getInitName()
  {
    return "<init>"; // do not really care
  }

  public boolean isAssignableFrom( Object toClazz, Object fromClazz )
  {
    Class t = (Class)toClazz;
    if( toClazz == Object.class ) // treating like Any
      return true;
    if( fromClazz == null )
      return !t.isPrimitive();
    Class f = (Class)fromClazz;
    if( t.isAssignableFrom( f ) )
      return true;

    boolean tp = t.isPrimitive();
    boolean fp = f.isPrimitive();

    if( !tp && !fp )
      return false;

    if( !tp )
      {
	t = wrapped( t );
	if( t == null )
	  return false;
      }

    if( !fp )
      {
	f = wrapped( f );
	if( f == null )
	  return false;
      }
    
    return primitiveAssignable( t, f );
  }

  public int getClassModifiers( Object clazz )
  {
    return ((Class)clazz).getModifiers();
  }

  public int getMethodModifiers( Object method )
  {
    return ((Method)method).getModifiers();
  }

  public int getFieldModifiers( Object field )
  {
    return ((Field)field).getModifiers();
  }

  /**
   *  Package is an opaque object, but must implement equals(Object) correctly.
   */
  public Object getPackage( Object clazz )
  {
    return "don't care"; // TODO: do it right in Java 1.2
  }

  public Enumeration getMethods( Object clazz, Object name, int nargs )
  {
    return new MethodList( (Class)clazz, (String)name, nargs );
  }

  public Object getMethodDeclaringClass( Object method )
  {
    return ((Method)method).getDeclaringClass();
  }

  public Object getReturnType( Object method )
  {
    return ((Method)method).getReturnType();
  }

  public Object[] getParameterTypes( Object method )
  {
    return ((Method)method).getParameterTypes();
  }

  public Object getField( Object clazz, Object name )
  {
    try
      {
	return ((Class)clazz).getDeclaredField( (String)name );
      }
    catch( NoSuchFieldException e )
      {
      }
    catch( SecurityException e )
      {
      }
    return null;
  }

  static class MethodList implements Enumeration
  {

    Method[]  methods;
    String    name;
    int       nargs;

    int p = 0;

    MethodList( Class clazz, String name, int nargs )
    {
      this.methods = clazz.getDeclaredMethods();
      this.name = name;
      this.nargs = nargs;
    }

    private void skip()
    {
      for( ; p < methods.length ; p++ )
	{
	  Method m = methods[p];
	  if( ! m.getName().equals( name ) )
	    continue;
	  Class[] types = m.getParameterTypes();
	  if( nargs < 0 )
	    {
	      if( types.length > 0 && types[types.length-1] == ArgList.class )
		return;
	    }
	  else
	    if( types.length == nargs )
	      return;
	}
    }

    public Object nextElement()
    {
      skip();
      return methods[p++];
    }

    public boolean hasMoreElements()
    {
      skip();
      return p < methods.length;
    }
  }

  public boolean isPrivateAccessible( Object clazz, Object accessorClass )
  {
    return clazz == accessorClass;
  }

  public boolean areSame( Object class1, Object class2 )
  {
    if( class1 == class2 )
      return true;

    Class c1 = (Class)class1;
    Class c2 = (Class)class2;

    boolean p1 = c1.isPrimitive();
    boolean p2 = c2.isPrimitive();

    if( !p1 && !p2 )
      return false;

    if( !p1 )
      c1 = wrapped( c1 );

    if( !p2 )
      c2 = wrapped( c2 );

    return c1 == c2;
  }

  static Class wrapped( Class clz )
  {
    if( clz == Integer.class )
      return intType;
    if( clz == Double.class )
      return doubleType;
    if( clz == Float.class )
      return floatType;
    if( clz == Character.class )
      return charType;
    if( clz == Short.class )
      return shortType;
    if( clz == Byte.class )
      return byteType;
    if( clz == Long.class )
      return longType;
    if( clz == Boolean.class )
      return booleanType;
    return null;
  }

  static boolean primitiveAssignable( Class to, Class from )
  {
    if( to == from )
      return true;

    if( to == byteType || to == booleanType || from == booleanType )
      return false;

    if( from == byteType || from == charType )
      return true;

    if( from == doubleType )
      return false;

    if( to == doubleType )
      return true;

    if( from == floatType )
      return false;

    if( to == longType )
      return true;

    if( from == longType )
      return false;

    if( to == intType )
      return true;

    // 'to' must be 'short' or 'char' -> cannot be assigned

    return false;
  }

}
