/*
    Copyright (c) Peter Sorotokin, 1998-2000
    See file "License.html" for terms of usage and
    redistribution.   
*/
package kmy.jint.util;

import java.util.Enumeration;

import kmy.jint.reflect.AbstractClass;
import kmy.jint.reflect.AbstractMethod;
import kmy.jint.reflect.AbstractField;
import kmy.jint.reflect.JintClassTable;
import kmy.jint.parser.Identifier;

public class JintClassMethods extends ClassMethods
{

  private JintClassTable classTable;

  public JintClassMethods( JintClassTable ct )
  {
    classTable = ct;
  }

  public Object getSuperclass( Object clazz )
  {
    return ((AbstractClass)clazz).getSuperclass();
  }

  public Object[] getImplementedInterfaces( Object clazz )
  {
    return ((AbstractClass)clazz).getImplementedInterfaces();
  }

  public Object getArgListClass()
  {
    return classTable.arglistClass;
  }

  public Object getAnyClass()
  {
    return classTable.anyClass;
  }

  /**
   *  Returns object used to name constructors ("<init>").
   */
  public Object getInitName()
  {
    return classTable.getNameTable().INIT;
  }

  public boolean isAssignableFrom( Object toClazz, Object fromClazz )
  {
    return classTable.canAssign( (AbstractClass)fromClazz, (AbstractClass)toClazz );
  }

  public int getClassModifiers( Object clazz )
  {
    return ((AbstractClass)clazz).getModifiers();
  }

  public int getMethodModifiers( Object method )
  {
    return ((AbstractMethod)method).getModifiers();
  }

  public int getFieldModifiers( Object field )
  {
    return ((AbstractField)field).getModifiers();
  }

  /**
   *  Package is an opaque object, but must implement equals(Object) correctly.
   */
  public Object getPackage( Object clazz )
  {
    return ((AbstractClass)clazz).getPackage();
  }

  public Enumeration getMethods( Object clazz, Object name, int nargs )
  {
    return ((AbstractClass)clazz).getMethods( (Identifier)name, nargs );    
  }

  public Object getMethodDeclaringClass( Object method )
  {
    return ((AbstractMethod)method).getDeclaringClass();
  }

  public Object getReturnType( Object method )
  {
    return ((AbstractMethod)method).getType();
  }

  public Object[] getParameterTypes( Object method )
  {
    return ((AbstractMethod)method).getParameterTypes();
  }

  public Object getField( Object clazz, Object name )
  {
    return ((AbstractClass)clazz).getField( (Identifier)name );
  }

  public boolean isPrivateAccessible( Object clazz, Object accessorClass )
  {
    return ((AbstractClass)accessorClass).getFullName().startsWith( 
				((AbstractClass)clazz).getFullName() );
  }

}
