/*
    Copyright (c) Peter Sorotokin, 1998-2000
    See file "License.html" for terms of usage and
    redistribution.   
*/
package kmy.test.java;

/**
 *  Testing areas: method call, overloading and method name resolution
 *  Tested on both javac and jintc on Thu Apr 29 23:35:32 PDT 1999.
 *  Tested on jintc on Wed Jun  2 22:55:56 PDT 1999.
 */
public class Call
{

  A java;

  static String println( String s )
  {
    return s;
  }

  public String toString()
  {
    return "Call";
  }

  class A
  {

    A lang;
    A System;
    Call out; // remains null

    A()
    {
      lang = this;
      System = this;
    }

    int method( int a )
    {
      if( a != 'I' )
	throw new RuntimeException( "Unexpected call" );
      return 'I';
    }

    int method( char a )
    {
      if( a != 'C' )
	throw new RuntimeException( "Unexpected call" );
      return 'C';
    }

    int method( byte a )
    {
      if( a != 'B' )
	throw new RuntimeException( "Unexpected call" );
      return (byte)'B';
    }

    int method( short a )
    {
      if( a != 'S' )
	throw new RuntimeException( "Unexpected call" );
      return (short)'S';
    }

    int method2( short a )
    {
      if( a != 'S' )
	throw new RuntimeException( "Unexpected call" );
      return (short)'S';
    }

    int method2( int a )
    {
      if( a != 'I' )
	throw new RuntimeException( "Unexpected call" );
      return 'I';
    }

    int method2( char a )
    {
      if( a != 'C' )
	throw new RuntimeException( "Unexpected call" );
      return 'C';
    }

    class Q
    {
      int method( int u )
      {
	return 10;
      }

      int test( char u )
      {
	return method( u );
      }
    }

    int testQ()
    {
      Q q = new Q();
      return q.method( '0' );
    }

  }

  class B extends A
  {
  }

  public static void main( String[] args )
  {
    boolean verbose = args.length == 0 || !args[0].equals( "-s" );
    (new Call()).test( verbose );
  }

  void test( boolean verbose )
  {

    try
      {
	A a = new A();

	java = a;

	int i = 'I';
	short s = (short)'S';
	byte b = (byte)'B';

	if( a.method('C') != 'C' )
	  System.out.println( "method(char): FAILED" );
	else
	  if( verbose )
	    System.out.println( "method(char): passed" );
	if( a.method( i ) != 'I' )
	  System.out.println( "method(int): FAILED" );
	else
	  if( verbose )
	    System.out.println( "method(int): passed" );
	if( a.method( b ) != 'B' )
	  System.out.println( "method(byte): FAILED" );
	else
	  if( verbose )
	    System.out.println( "method(byte): passed" );
	if( a.method( s ) != 'S' )
	  System.out.println( "method(short): FAILED" );
	else
	  if( verbose )
	    System.out.println( "method(short): passed" );

	if( a.method2('C') != 'C' )
	  System.out.println( "method2(char): FAILED" );
	else
	  if( verbose )
	    System.out.println( "method2(char): passed" );
	if( a.method2( i ) != 'I' )
	  System.out.println( "method2(int): FAILED" );
	else
	  if( verbose )
	    System.out.println( "method2(int): passed" );
	b = (byte)'S';
	if( a.method2( b ) != 'S' )
	  System.out.println( "method2(byte): FAILED" );
	else
	  if( verbose )
	    System.out.println( "method2(byte): passed" );
	if( a.method2( s ) != 'S' )
	  System.out.println( "method2(short): FAILED" );
	else
	  if( verbose )
	    System.out.println( "method2(short): passed" );

	if( a.testQ() != 10 )
	  System.out.println( "testQ: FAILED" );
	else
	  if( verbose )
	    System.out.println( "testQ: passed" );

	if( java.lang.System.out.println( "TEST" ) != "TEST" )
	  System.out.println( "field is more then package: FAILED" );
	else
	  if( verbose )
	    System.out.println( "field is more then package: passed" );

	java.lang.System = null;

	try
	  {
	    java.lang.System.out.println( "TEST" );
	    System.out.println( "Static method access: FAILED" );
	  }
	catch( NullPointerException e )
	  {
	    System.out.println( "Static method access: passed" );
	  }

	println( "TEST" ); // check if compiles and runs without errors

	if( ! toString().equals( "Call" ) )
	  System.out.println( "this.toString(), not self.toString(): FAILED" );
	else
	  if( verbose )
	    System.out.println( "this.toString(), not self.toString(): passed" );
      }
    catch( Exception e )
      {
	e.printStackTrace();
      }
  }
}
