/*
    Copyright (c) Peter Sorotokin, 1998-2000
    See file "License.html" for terms of usage and
    redistribution.   
*/
package kmy.test.java;

/**
 *  This class tests various java language statements
 */
public class Statements
{

  boolean verbose;

  public void assert( boolean cond, String id )
  {
    if( !cond )
      System.out.println( id + ": FAILED" );
    else
      if( verbose )
	System.out.println( id + ": passed" );
  }

  //--------- FOR ----

  public void forTest()
  {
    int S = 0;
    for( int i = 0 ; i <= 10 ; i++ )
      S += i*i;
    assert( S == 385, "forTest1" );

    S = 0;
    for( int i = 0 ; i <= 10 ; i++ )
      {
	if( i == 5 )
	  continue;
	S += i*i;
      }
    assert( S == 360, "forTest2" );

    S = 0;
    for( int i = 0 ; i <= 10 ; i++ )
      {
	if( i == 10 )
	  continue;
	S += i*i;
      }
    assert( S == 285, "forTest3" );

    S = 0;
    for( int i = 5 ; i <= 10 ; i++ )
      {
	if( i == 7 )
	  break;
	S += 3*i;
      }
    assert( S == 33, "forTest4" );
  }

  //-------------- SWITCH ---------------------

  void switchTest()
  {
    boolean fallThrough = false;
    for( int i = 0 ; i < 10 ; i++ )
      {
	switch( i )
	  {
	    case 0 :
	      if( i != 0 )
		System.out.println( "testSwitch1 : FAILED" );
	      else
		if( verbose )
		  System.out.println( "testSwitch1 : passed" );
	      break;
	    case 2 :
	      if( i != 2 )
		System.out.println( "testSwitch2 : FAILED" );
	      else
		if( verbose )
		  System.out.println( "testSwitch2 : passed" );
	      break;
	    case 4 :
	      if( i != 4 )
		System.out.println( "testSwitch3 : FAILED" );
	      else
		if( verbose )
		  System.out.println( "testSwitch3 : passed" );
	      break;
	    case 6 :
	      if( i != 6 )
		System.out.println( "testSwitch4 : FAILED" );
	      else
		if( verbose )
		  System.out.println( "testSwitch4 : passed" );
	    case 8 :
	      if( i == 6 )
		fallThrough = true;
	      if( i != 6 && i != 8 )
		System.out.println( "testSwitch5 [" + i + "]: FAILED" );
	      else
		if( verbose )
		  System.out.println( "testSwitch5 [" + i + "]: passed" );
	      break;
	    default :
	      if( i % 2 == 0 )
		System.out.println( "testSwitch6 [" + i + "]: FAILED" );
	      else
		if( verbose )
		  System.out.println( "testSwitch6 [" + i + "]: passed" );
	      break;
	  }
      }
    if( fallThrough )
      {
	if( verbose )
	  System.out.println( "testSwitch7: passed" );
      }
    else
      System.out.println( "testSwitch7: FAILED" );
  }

  //-------------- SYNCHRONIZED ----------------

  static int synctest( int index, Object lock )
  {
    int i;
    for( i = 1 ; i < 10 ; i++ )
      {
        synchronized( lock )
	  {
	    lock.notify(); // verify that it is locked
	    if( index == 0 )
	      return 0;
	    if( index == 1 )
	      break;
	    if( index == 2 )
	      continue;
	  }
	return -100;
      }
    return i;
  }

  static int runsynctest( int index )
  {
    Object lock = new Integer( index );
    int res = synctest( index, lock );
    try
      {
	lock.notify();
      }
    catch( IllegalMonitorStateException e )
      {
        return res;
      }
    return -2;
  }

  public void synchronizedTest()
  {
    int[] expected = { 0, 1, 10, -100 };
    for( int i = 0 ; i < 4 ; i++ )
      {
	int res = runsynctest(i);
	boolean failed = expected[i] != res;
	if( verbose || failed )
          {
	    if( failed )
	      {
	        System.out.print( "syncTest"+(i+1)+": FAILED" );
                if( verbose )
		  System.out.print( " (expected: " + expected[i] + " had: " + res + ")" ); 
		System.out.println();
              }
	    else
	      System.out.println( "syncTest"+(i+1)+": passed" );
          }
      }
  }

  //------------------- TRY/THROW --------------------

  static void throwIt( RuntimeException e )
  {
    throw e;
  }

  public void tryTest()
  {
    int a = 8;
    int b = 6;
    RuntimeException e1 = null;
    RuntimeException e2 = null;

    try
      {
        if( ++b == 7 )
	  {
	    e1 = new RuntimeException( "Ugu" );
	    throwIt( e1 );
	  }
	a = 10;
      }
    catch( RuntimeException e )
      {
	e2 = e1;
      }
    finally
      {
	b = 10;
      }

    assert( a == 8 && b == 10 && e1 == e2, "tryTest1" );
  }

  //------------------- WHILE ------------------------

  public void whileTest()
  {
    int S = 0;
    int i = 1;
    while( i <= 10 )
      S += i*(i++);
    assert( S == 385, "whileTest1" );

    S = 0;
    i = 0;
    while( i <= 10 )
      {
	if( i == 2 )
	  {
	    i++;
	    continue;
	  }
	S += i*(i++);
      }
    assert( S == 381, "whileTest2" );
  }

  public static void main( String[] args )
  {
    Statements s = new Statements();
    s.verbose = args.length == 0 || args[0].indexOf( 'v' ) >= 0;
    s.forTest();
    s.switchTest();
    s.synchronizedTest();
    s.tryTest();
    s.whileTest();
  }
}




