/*
    Copyright (c) Peter Sorotokin, 1998-2000
    See file "License.html" for terms of usage and
    redistribution.   
*/
package kmy.test.jint;
import java.io.*;
import java.util.*;

public class AnyType
{

  public int field;

  public int empty( int n )
  {
    return n;
  }

  public static void perf( boolean verbose ) throws Exception
  {
    int i;
    int nCalls = 5000;
    int nCalls1 = 50*nCalls;
    long t = System.currentTimeMillis();
    for( i = 0 ; i < nCalls1 ; i++ )
      {
	Object[] arr = new Object[2];
	arr[0] = new Integer( 10 );
      }

    var res = (double)(System.currentTimeMillis() - t)/nCalls1;
    if( verbose )
      {
	System.out.println( `{Integer(10),null} creation time: ${res%6.4f} millisec` );
      }

    var anyNum = 20;
    var emptyStr = "";
    var anyObj = System.out;
    t = System.currentTimeMillis();
    for( i = 0 ; i < nCalls ; i++ )
      anyObj.print( emptyStr );
    res = (double)(System.currentTimeMillis() - t)/nCalls;
    if( verbose )
      {
	System.out.println( "Average times per call" );
        System.err.println( `System.out.print("") [soft]: ${res%6.4f} millisec` );
      }
    else
      System.out.println( res );

    PrintStream out = System.out;
    String s = "";
    t = System.currentTimeMillis();
    for( i = 0 ; i < nCalls1 ; i++ )
      out.print( s );
    res = (double)(System.currentTimeMillis() - t)/nCalls1;
    if( verbose )
      System.err.println( `System.out.print("") [hard]: ${res%6.4f} millisec` );

    anyObj = new AnyType();
    t = System.currentTimeMillis();
    for( i = 0 ; i < nCalls ; i++ )
      anyObj.empty( anyNum );
    res = (double)(System.currentTimeMillis() - t)/nCalls;
    if( verbose )
      System.err.println( `Empty function [soft]: ${res%6.4f} millisec` );
    else
      System.out.println( res );

    AnyType tt = anyObj;
    int n = anyNum;

    t = System.currentTimeMillis();
    for( i = 0 ; i < nCalls1 ; i++ )
      {
        tt.empty( n );
      }
    res = (double)(System.currentTimeMillis() - t)/nCalls1;
    if( verbose )
      System.err.println( `Empty function [hard]: ${res%6.4f} millisec` );

    t = System.currentTimeMillis();
    for( i = 0 ; i < nCalls ; i++ )
      n = anyObj.field;
    res = (double)(System.currentTimeMillis() - t)/nCalls;
    if( verbose )
      System.err.println( `Field access [soft]: ${res%6.4f} millisec` );
    else
      System.out.println( res );
    
    t = System.currentTimeMillis();
    for( i = 0 ; i < nCalls1 ; i++ )
      n = tt.field;
    res = (double)(System.currentTimeMillis() - t)/nCalls1;
    if( verbose )
      System.err.println( `Field access [hard]: ${res%6.4f} millisec` );

    t = System.currentTimeMillis();
    for( i = 0 ; i < nCalls ; i++ )
      anyNum = anyNum*anyNum*anyNum*anyNum*anyNum*
		anyNum*anyNum*anyNum*anyNum*anyNum*anyNum;
      
    res = (double)(System.currentTimeMillis() - t)/(10*nCalls);
    if( verbose )
      System.err.println( `Multiplication operation [soft]: ${res%6.4f} millisec` );
    else
      System.out.println( res );

    t = System.currentTimeMillis();
    for( i = 0 ; i < nCalls1 ; i++ )
      n = n*n*n*n*n*n*n*n*n*n*n;
    res = (double)(System.currentTimeMillis() - t)/(10*nCalls1);
    if( verbose )
      System.err.println( `Multiplication operation [hard]: ${res%6.4f} millisec` );
  }

  public static void test( boolean verbose ) throws Exception
  {

    Any var = 'A';
    if( var.getClass() == Character.class && var == 'A' && var > 10 && var < 'B' )
      {
	if( verbose )
	  System.out.println( "Character: passed" );
      }
    else
      {
	System.out.println( "Character: FAILED" );
	System.out.println( "\tclass: " + var.getClass() );
	System.out.println( "\tvalue: " + var );
      }

    var = 10;
    if( var.getClass() == Integer.class && var == 10 && var > 9 && var < 11.0 )
      {
	if( verbose )
	  System.out.println( "Integer: passed" );
      }
    else
      {
	System.out.println( "Integer: FAILED" );
	System.out.println( "\tclass: " + var.getClass() );
	System.out.println( "\tvalue: " + var );
      }

    var = 10.0;
    if( var.getClass() == Double.class && var == 10.0 && var > 9.0 && var < 11 )
      {
	if( verbose )
	  System.out.println( "Double: passed" );
      }
    else
      {
	System.out.println( "Double: FAILED" );
	System.out.println( "\tclass: " + var.getClass() );
	System.out.println( "\tvalue: " + var );
      }

    Any table = new Hashtable();
    
    table["key"] = "value";

    if( table["key"] == "value" )
      {
	if( verbose )
	  System.out.println( "Hashtable: passed" );
      }
    else
      System.out.println( "Hashtable: failed" );


    table = new Vector();

    table[0] = "elem";

    if( table[0] == "elem" )
      {
	if( verbose )
	  System.out.println( "Vector: passed" );
      }
    else
      System.out.println( "Vector: failed" );
  }

  public static void main( String[] args ) throws Exception
  {
    boolean verbose = args.length == 0 || args[0].indexOf( 'v' ) >= 0;
    boolean perf = args.length == 0 || args[0].indexOf( 'p' ) >= 0;
    AnyType.test( verbose );
    if( perf )
      {
        AnyType.perf( verbose );
      }
  }
}


