/*
    Copyright (c) Peter Sorotokin, 1998-2000
    See file "License.html" for terms of usage and
    redistribution.   
*/
package kmy.test.jint;

import kmy.regex.util.*;
import java.util.*;

public class Calculator
{

  var ident = new RE.Identifier();
  var integer = new RE.Integer();
  var real = new RE.Float();
  var operator = /[-+*\/=]|\>\>\>?|\<\</S;

  var opPriorities = new Hashtable();

  var vars = new Hashtable();

  opPriorities["="] = 3;
  opPriorities[">>"] = 4;
  opPriorities[">>>"] = 4;
  opPriorities["<<"] = 4;
  opPriorities["+"] = 5;
  opPriorities["-"] = 5;
  opPriorities["*"] = 6;
  opPriorities["/"] = 6;
  opPriorities["M"] = 10; // unary minus
  opPriorities[")"] = 0;
  opPriorities["\n"] = 0;
  opPriorities["("] = 0;

  eval( input )
  {
    var expectOp = false;
    var operators = new Stack();
    var values = new Stack();
    var done = false;

    Sys.print( "calc: " );
    Sys.flush();

    while( !done )
      {
	if( expectOp )
	  {
	    var op;

	    expectOp = false;

	    if( input.read( "\n" ) || input.read( "\r\n" ) || input.read( ";" ) )
	      {
		op = "\n";
		done = true;
	      }
	    else if( input.read( operator ) )
	      op = operator.getMatchString();
	    else if( input.read( ")" ) )
	      {
		expectOp = true;
		op = ")";
	      }
	    else
	      throw new RuntimeException( "Unknown operator: " + (char)input.read() );

	    var pri = opPriorities[op];

	    if( pri == 3 ) // '=' priority
	      pri--;

	    while( !operators.empty() && opPriorities[operators.peek()] >= pri )
	      {
		var evalOp = operators.pop();
		// Sys.println( "op " + evalOp );
		if( pri == 0 && evalOp == "(" )
		  break;

		var value2 = values.pop();
		var result;

		if( evalOp == "M" )
		  {
		    result = -value2;
		  }
		else
		  {
		    var value1 = values.pop();
		    if( value2 instanceof String )
		      value2 = vars[value2];
		    if( !evalOp.equals( "=" ) && value1 instanceof String )
		      value1 = vars[value1];
		    switch( evalOp )
		      {
		        case "=" :
			  result = value2;
			  vars[value1] = value2;
			  break;
		        case ">>" :
			  result = value1 >> value2;
			  break;
		        case ">>>" :
			  result = value1 >>> value2;
			  break;
		        case "<<" :
			  result = value1 << value2;
			  break;
		        case "+" :
			  result = value1 + value2;
			  break;
		        case "-" :
			  result = value1 - value2;
			  break;
		        case "*" :
			  result = value1 * value2;
			  break;
		        case "/" :
			  result = value1 / value2;
			  break;
		        default:
			  throw new RuntimeException( "Undefined op: '" + evalOp + "' " +
						    evalOp.getClass() );
		      }
		    // Sys.println( value1 + (String)evalOp + value2 + "->" + result );
		  }
		values.push( result );
	      }
	    if( !done && pri != 0 )
	      operators.push( op );
	  }
	else
	  {
	    // skip empty lines
	    while( input.read( "\n" ) || input.read( "\r" ) )
	      ;

	    if( input.read( real ) )
	      values.push( real.getMatch().doubleValue() );
	    else if( input.read( integer ) )
	      values.push( integer.getMatch().longValue() );
	    else if( input.read( ident ) )
	      values.push( ident.getMatchString() );
	    else if( input.read( "(" ) )
	      {
		operators.push( "(" );
		continue;
	      }
	    else if( input.read( "-" ) )
	      {
		operators.push( "M" );
		continue;
	      }
	    else
	      throw new RuntimeException( "Unknown value" );
	    expectOp = true;
	  }
      }
    if( values.size() > 0 )
      {
	var v = values.pop();
	Sys.println( v instanceof String ? vars[v] : v );
      }
  }

  public static void main( String[] args )
  {
    try
      {
	var calc = new Calculator();
	var input = Sys.getStdin();
	input.setIgnore( /[ \t]+/ );
	while( true )
	  calc.eval( input );
      }
    catch( Exception e )
      {
	e.printStackTrace();
      }
  }
}
