/*
    Copyright (c) Peter Sorotokin, 1998-2000
    See file "License.html" for terms of usage and
    redistribution.   
*/
package kmy.test.jint;

import java.io.*;
import java.util.*;
import kmy.jint.io.*;
import kmy.regex.util.*;

public class ReaderTest
{

  PipedReader rawIn;
  PipedWriter rawOut;

  int seed = (int)System.currentTimeMillis();

  ReaderTest()
  {
    try
      {
	rawIn = new PipedReader();
	rawOut = new PipedWriter( rawIn );
      }
    catch( IOException e )
      {
	e.printStackTrace();
      }
  }

  String nextWord( Random random )
  {
    int r = random.nextInt() & 0x3FF;
    int length = 1 + r;
    char[] str = new char[length];
    for( int i = 0 ; i < length ; i++ )
      {
	r = (random.nextInt() & 0x7FFFFFFF)%52;
	str[i] = (char)( r >= 26 ? 'A' + (r-26) : 'a' + r );
      }
    return new String( str, 0, length );
  }

  Thread receiver
  {
    public void run()
    {
      Random random = new Random( seed );
      Regex word = new RE.Word();
      try
	{
	  JintReader in = new JintReader( rawIn );
	  in.setIgnore( new RE.Float() );
	  while( true )
	    {
	      String wstr = nextWord( random );
	      if( ! in.read( word ) )
		throw new Exception( "Could not read stream!" );
	      String rstr = word.getMatchString();
	      if( ! rstr.equals( wstr ) )
		throw new Exception( "Mismatch: [" + rstr + "] [" + wstr + "]" );
	      Sys.println( "Match OK [" + rstr.length() + "]" );
	    }
	}
      catch( Exception e )
	{
	  e.printStackTrace();
	}
      System.exit(0);
    }
  }

  Thread sender
  {
    public void run()
      {
	Random random = new Random( seed );
	Random random1 = new Random( seed );
	try
	  {
	    PrintWriter out = new PrintWriter( rawOut );
	    while( true )
	      {
		String word = nextWord( random );
		out.print( word );
		out.print( random1.nextInt() );
		out.print( '.' );
		out.print( random1.nextInt()&0x7FFFFFFF );
	      }
	  }
	catch( Exception e )
	  {
	    e.printStackTrace();
	  }
	System.exit(0);
      }
  }

  public static void main( String[] args )
  {
    ReaderTest t = new ReaderTest();
    t.sender.start();
    t.receiver.start();
  }
}
