#!/usr/local/bin/perl

open( TEST, "regex.txt" ) || die "Cannot open regex.txt!\n";

if( $ARGV[0] =~ /^-.*v/ ) {
  $verbose=1;
}

if( $ARGV[0] =~ /^-.*p/ ) {
  $perf=1;
}

if( $ARGV[0] =~ /^-.*g/ ) {
  $gen=1;
}

$total=0;

if( $gen ) {
  print( "public class ReTest\n" );
  print( "{\n" );
  print( "  static int N = 1;" );
  print( "  public static void main( String[] args )\n" );
  print( "  {\n" );
  print( "    String[] test = new String[100];\n" );
  print( "    boolean[] expect = new boolean[100];\n" );
  print( "    boolean verbose = args.length > 0 && args[0].indexOf( 'v' ) > 0;\n" );
  print( "    boolean perf = args.length > 0 && args[0].indexOf( 'p' ) > 0;\n" );
  print( "    N = ( perf ? 10000 : 1 );\n" );
  print( "    double total = 0.0;\n" );
}

$no = 0;

sub genCompare
{
    print "\n";
    print "    if( verbose )\n";
    print "      System.out.println( \"--------  $regex \" );\n";
    print "    for( int i = 0 ; i < $no ; i++ )\n";
    print "      {\n";
    print "        CharString t = new CharString( test[i] );\n";
    print "        long time1 = System.currentTimeMillis();\n";
    print "        boolean passed = true;\n";
    print "        for( int j = 0 ; j < N ; j++ )\n";
    print "	     if( (t ~ \`$regex\`) != expect[i] )\n";
    print "	       passed = false;\n";
    print "	   if( !passed )\n";
    print "          System.out.println( t + \": FAILED (\" + !expect[i] + \")\" );\n";
    print "        else\n";
    print "          if( verbose && !perf )\n";
    print "            System.out.println( t + \": passed (\" + expect[i] + \")\" );\n";
    print "        long time2 = System.currentTimeMillis();\n";
    print "        double average = (time2 - time1)/(double)N;\n";
    print "        total += average;\n";
    print "        if( N > 1 )\n";
    print "          System.out.println( (verbose? t + \": \": \"\" ) + average );\n";
    print "      }\n";
    $no = 0;
}

while( <TEST> )
{
  next if( /^\s*#/ );
  chop;
  next if( !$_ );
  if( /^;/ ) {
    ($dummy,$test,$res,$v1,$v2,$v3,$v4) = split( /;/ );
    if( $gen ) {
      print( "    test[$no] = \"$test\";\n" );
      print( "    expect[$no] = $res;\n" );
      $no++;
      next;
    }
    if( $perf ) {
      $rep=10000;
      $code = "\$start=(times)[0];for(\$i=0;\$i<$rep;\$i++){\$test=~/$regex/};\$end=(times)[0];";
      eval( $code );
      $res=1000*($end-$start)/$rep;
      if( $verbose ) {
        printf( "$test:\t%6.4f\n", $res );
      } else {
        printf( "%6.4f\n", $res );
      }
      $total += $res;
      next;
    }
    if( $verbose ) {
      print "Testing: '$test' =~ /$regex/ [$res]\n";
    }
    if( $test =~ /$regex/ ) {
      if( $res eq "true" && $1 eq $v1 && 
	  $2 eq $v2 && $3 eq $v3 && $4 eq $v4 ) {
	print( "\tpassed $1 $v1\n" ) if ( $verbose );
      } else {
	print "Testing: '$test' =~ /$regex/ [$res]\n" if ( ! $verbose );
	print( "\tFAILED\n" );
	print( "\t\$1=$1 \$v1=$v1\n" ) if($1);
	print( "\t\$2=$2 \$v2=$v2\n" ) if($2);
	print( "\t\$3=$3 \$v3=$v3\n" ) if($3);
	print( "\t\$4=$4 \$v4=$v4\n" ) if($4);
      }
    } else {
      if( $res == "false" ) {
	print( "\tpassed\n" ) if ($verbose );
      } else {
	print "Testing: '$test' =~ /$regex/ [$res]\n" if ( ! $verbose );
	print( "\tFAILED" );
      }
    }
  } else {
    &genCompare() if( $gen && $no > 0);
    $regex=$_;
    print "-------- $regex\n" if( $verbose );
    print "\n" if ($gen );
  }
}

if( $gen ) {
  &genCompare() if( $no > 0 );
  print "     if( N > 1 )\n";
  print "       System.out.println( \"Total: \" + total );\n";
  print( "  }\n" );
  print( "}\n" );
}
printf( "--------------\nTotal: %6.4f\n", $total ) if ( $perf );
